 /*
 * 20160215 SSI Encoder Test.c
 *
 * Created: 15-2-2016 13:23:37
 * Author : John
 */ 

//	F_CPU defined in toolchain -> AVR/GNU C Compiler -> Symbols -> undefined symbols -> F_CPU16000000UL

//	includes
#include <avr/io.h>
#include <util/delay.h>

 
#include <stdio.h>


//	defines
#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

//	#define LED3 PIND2

//	define SSI interface
#define EncDataBit			PIND2		//	PD2 pin 2 of port D input
#define EncClock			PORTD3		//	PD3 pin 3 of port D output

//	define LCD interface
#define lcd_D7_port			PORTD		//	lcd d7 connection
#define lcd_D7_bit			PORTD7
#define lcd_D7_ddr			DDRD

#define lcd_D6_port			PORTD		//	lcd d6 connection
#define lcd_D6_bit			PORTD6
#define lcd_D6_ddr			DDRD

#define lcd_D5_port			PORTD		//	lcd d5 connection
#define lcd_D5_bit			PORTD5
#define lcd_D5_ddr			DDRD

#define lcd_D4_port			PORTD		//	lcd d4 connection
#define lcd_D4_bit			PORTD4
#define lcd_D4_ddr			DDRD

#define lcd_E_port			PORTB		//	lcd enable connection
#define lcd_E_bit			PORTB1
#define lcd_E_ddr			DDRB

#define lcd_RS_port			PORTB		//	lcd enable connection
#define lcd_RS_bit			PORTB0
#define lcd_RS_ddr			DDRB

//	LCD module info
#define lcd_LineOne			0x00		//	start of line 1
#define lcd_LineTwo			0x40		//	start of line 2

//	LCD instructions
#define lcd_Clear			0b00000001
#define lcd_Home			0b00000010
#define lcd_EntryMode		0b00000110
#define lcd_DisplayOff		0b00001000
#define lcd_DisplayOn		0b00001100
#define lcd_FunctionReset	0b00110000
#define lcd_FunctionSet4bit	0b00101000
#define lcd_SetCursor		0b10000000

//	start up text
char company[]	="JG Electronics";
char version[]	="SSI Encoder 2.0";
char date[]		="20160215";

//	global variables

//	function prototypes



void lcd_init_4d(void);
void lcdOn(void);
void lcdOff(void);
void lcdClear(void);
void lcdHome(void);
void lcd_write_instruction_4d(uint8_t);
void lcd_write_string_4d(char*);
void lcd_write_character_4d(uint8_t);
void lcd_write_4d(uint8_t);

//	interrupts

//	main
int main(void)
{
//	inits
//	Data direction Register Port D
//	DDRD = 0b00001000;				//	set pin3 (bit 3 van DDRD) PIND3 as output and pin2 (bit 2 van DDRD) PIND2 as input
//	PORTD  |= (1 << EncDataBit);	//	set pullup resistor on PIND2
//	also possible to use
//	DDRD = 0X04;					//	set pin2 (bit 2) PIND2 as output with hexadecimal number

//	configure the microprocesor pins for the data lines of the lcd
/*	lcd_D7_ddr |= (1<<lcd_D7_bit);	//	4 data lines - output
	lcd_D6_ddr |= (1<<lcd_D6_bit);
	lcd_D5_ddr |= (1<<lcd_D5_bit);
	lcd_D4_ddr |= (1<<lcd_D4_bit);
*/
//	faster to configure all ports of DDRD at once. Bit at 0 is input and bit at 1 is output
	DDRD = 0b11111000;
//	DDRD = 0b00001000;


//	configure the microprocesor pins for the control lines of the lcd
//	lcd_E_ddr |= (1<<lcd_E_bit);		//	E line - output
//	lcd_RS_ddr |= (1<<lcd_RS_bit);		//	RS line - output
//	faster to configure all ports of DDRB at once. Bit at 0 is input and bit at 1 is output
	DDRB = 0b00000011;

//	initialize the LCD controller as determined by the defines (LCD instructions)
	lcd_init_4d();

//	display program information
	lcd_write_string_4d(company);							//	display first line of information
	lcd_write_instruction_4d(lcd_SetCursor | lcd_LineTwo);	//	set cursor to start of line 2
	_delay_us(120);											//	40 us delay (min)
	lcd_write_string_4d(version);							//	display second line of information
	_delay_ms(5000);

//	event loop
    while (1) 
    {
//	***************************************************************************************************
//	Port test output
	/*
		PORTD |= (1 << PIND2);	//	set output PIND2 of PORTD high (1)
		_delay_ms(2000);		//	delay 2 sec
		PORTD &= ~(1 << PIND2);	//	set output PIND2 of PORTD low (0)
		_delay_ms(2000);		//	delay 2 sec
	*/
	/*
	//	also possible to do the same with a toggle
		PORTD ^= (1 << PIND2);	//	toggle output of PIND2 of PORTD
		_delay_ms(2000);		//	delay 2 sec
	*/
	/*
	//	is it more clear to use LED3 instead of PIND2 (Pin 2 Port D) then #define LED3 PIND2
		PORTD ^= (1 << LED3);	//	toggle output of of PIND2 (PD2) of PORTD
		_delay_ms(2000);		//	delay 2 sec
	*/
//	***************************************************************************************************


//	Port test maximum output of freq of block wave encoder test
		int			count;
		uint32_t	encData = 0;
		uint32_t	binData = 0;
		
		encData = 0;
		PORTD &= ~(1 << EncClock);	//	set output PD3 of PORTD low (0) Encoder Clock
		_delay_us(10);
		for(count = 0; count < 25; count++) {
			PORTD |= (1 << EncClock);	//	set output PD3 of PORTD high (1) Encoder Clock
			_delay_us(5);
			if(bit_is_clear(PIND,EncDataBit)) {
				encData = encData << 1;
			} else {
				encData = encData << 1;
				encData = encData + 1;
			}
			_delay_us(5);
			PORTD &= ~(1 << EncClock);	//	set output PD3 of PORTD low (0) Encoder Clock
			_delay_us(10);
		}
		PORTD |= (1 << EncClock);	//	set output PD3 of PORTD low (0) Encoder Clock

//	convert Gray code to binairy data
		bitWrite(binData, 25, bitRead(encData, 25));
		for(count = 24; count != 0; count--) {
			if(bitRead(binData, count + 1) ^ bitRead(encData, count)) {
				bitWrite(binData, count, 1);
			} else {
				bitWrite(binData, count, 0);
			}
			if(bitRead(binData, 1) ^  bitRead(encData, 0)) {
				bitWrite(binData, 0, 1);
			} else {
				bitWrite(binData, 0, 0);
			}
		}

//	output on LCD 
		lcdClear();
		lcdHome();
		char str[16];
		sprintf(str, "test %li", binData);
		lcd_write_string_4d(str);
		_delay_ms(100);
	}
}

//	***************************************************************************************************
//	LCD functions

void lcd_init_4d(void) {
	_delay_ms(500);													//	power up delay
	
//	set up the RS and E lines for the 'lcd_write_4' subroutine
	lcd_RS_port &= ~(1 << lcd_RS_bit);								//	select LCD instruction register (RS low)
	lcd_E_port &= ~(1 << lcd_E_bit);								//	make sure E is initialy low
	
//	reset the LCD controller
	lcd_write_4d(lcd_FunctionReset);								//	first part of reset sequence
	_delay_ms(10);													//	4.1 ms delay (min)
	
	lcd_write_4d(lcd_FunctionReset);								//	second part of reset sequence
	_delay_us(200);													//	100 us delay (min)
	
	lcd_write_4d(lcd_FunctionReset);								//	third part of reset sequence
	_delay_us(200);													//	this delay is omitted in the data sheet
	
	lcd_write_4d(lcd_FunctionSet4bit);								//	set 4 bit mode
	_delay_us(80);													//	40 us delay (min)
	
	lcd_write_instruction_4d(lcd_FunctionSet4bit);					//	set modes, lines and font
	_delay_us(80);													//	40 uS delay (min)
	
	lcd_write_instruction_4d(lcd_EntryMode);						//	set desired shift characteristics
	_delay_us(80);													//	40 us delay (min)
	
	lcdOff();
	lcdClear();
	lcdOn();
}


void lcdOn() {
	lcd_write_instruction_4d(lcd_DisplayOn);						//	turn display on
	_delay_us(80);													//	40 uS delay (min)
}

void lcdOff() {
	lcd_write_instruction_4d(lcd_DisplayOff);						//	turn display off
	_delay_us(80);													//	40 us delay (min)
}

void lcdClear() {
	lcd_write_instruction_4d(lcd_Clear);							//	clear display
	_delay_ms(4);													//	1.6 ms delay (min)
}

void lcdHome() {
	lcd_write_instruction_4d(lcd_Home);								//	clear display
	_delay_ms(4);													//	1.6 ms delay (min)
}

void lcd_write_instruction_4d(uint8_t theInstruction) {
	lcd_RS_port &= ~(1 << lcd_RS_bit);								//	select LCD instruction register (RS low)
	lcd_E_port &= ~(1 << lcd_E_bit);								//	make sure E is initialy low
	lcd_write_4d(theInstruction);									//	write the upper 4 bits of the data
	lcd_write_4d(theInstruction << 4);								//	write the lower 4 bits of the data
}

void lcd_write_string_4d(char theString[]) {
	volatile int i = 0;												//	character counter
	
	while(theString[i] != 0) {
		lcd_write_character_4d(theString[i]);
		i++;
		_delay_us(120);													//	40 us delay (min)
	}
}

void lcd_write_character_4d(uint8_t theData) {
	lcd_RS_port |= (1 << lcd_RS_bit);								//	select LCD data register (RS high)
	lcd_E_port &= ~(1 << lcd_E_bit);								//	make sure E is initialy low
	lcd_write_4d(theData);											//	write the upper 4 bits of the data
	lcd_write_4d(theData << 4);										//	write the lower 4 bits of the data
}

void lcd_write_4d(uint8_t theByte) {
	
	lcd_D7_port &= ~(1 << lcd_D7_bit);								//	assume that data is '0'
	if(theByte & 1 << 7) lcd_D7_port |= (1 << lcd_D7_bit);			//	make data '1' if necessary
	
	lcd_D6_port &= ~(1 << lcd_D6_bit);								//	assume that data is '0'
	if(theByte & 1 << 6) lcd_D6_port |= (1 << lcd_D6_bit);			//	make data '1' if necessary
	
	lcd_D5_port &= ~(1 << lcd_D5_bit);								//	assume that data is '0'
	if(theByte & 1 << 5) lcd_D5_port |= (1 << lcd_D5_bit);			//	make data '1' if necessary
	
	lcd_D4_port &= ~(1 << lcd_D4_bit);								//	assume that data is '0'
	if(theByte & 1 << 4) lcd_D4_port |= (1 << lcd_D4_bit);			//	make data '1' if necessary
	
//	write the data
	lcd_E_port |= (1 << lcd_E_bit);									//	set LCD enable high
	_delay_us(2);													//	implement 'data setup time' (80ns) and 'enable pulse width' (230ns)
	lcd_E_port &= ~(1 << lcd_E_bit);								//	LCD enable low
	_delay_us(2);													//	implement 'data hold time' (10ns) and 'enable cycle time' (500ns)	
}



